@@ -30,6 +30,7 @@ from consts import (
     KEY_UUID,
 )
 from async_preparers import app_config_aad_decorator_async
+from devtools_testutils import set_custom_default_matcher
 from devtools_testutils.aio import recorded_by_proxy_async
 import pytest
 import copy
@@ -219,6 +220,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_key_label(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         items = await self.convert_to_list(self.client.list_configuration_settings(label_filter=LABEL, key_filter=KEY))
         assert len(items) == 1
@@ -228,6 +231,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_only_label(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         items = await self.convert_to_list(self.client.list_configuration_settings(label_filter=LABEL))
         assert len(items) == 1
@@ -237,6 +242,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_only_key(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         items = await self.convert_to_list(self.client.list_configuration_settings(key_filter=KEY))
         assert len(items) == 2
@@ -246,6 +253,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_fields(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         items = await self.convert_to_list(
             self.client.list_configuration_settings(key_filter="*", label_filter=LABEL, fields=["key", "content_type"])
@@ -257,6 +266,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_reserved_chars(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         async with self.create_aad_client(appconfiguration_endpoint_string) as client:
             reserved_char_kv = ConfigurationSetting(key=KEY, label=LABEL_RESERVED_CHARS, value=TEST_VALUE)
             reserved_char_kv = await client.add_configuration_setting(reserved_char_kv)
@@ -266,10 +277,12 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
             assert all(x.label == LABEL_RESERVED_CHARS for x in items)
             await client.delete_configuration_setting(reserved_char_kv.key)
 
-    # NOTE: Label filter does not support wildcard at beginning on filters. https://docs.microsoft.com/en-us/azure/azure-app-configuration/rest-api-key-value#supported-filters
+    # NOTE: Label filter does not support wildcard at beginning on filters. https://docs.microsoft.com/azure/azure-app-configuration/rest-api-key-value#supported-filters
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_contains(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         items = await self.convert_to_list(self.client.list_configuration_settings(label_filter=LABEL + "*"))
         assert len(items) == 1
@@ -279,6 +292,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_correct_etag(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         async with self.create_aad_client(appconfiguration_endpoint_string) as client:
             to_list_kv = self.create_config_setting()
             await self.add_for_test(client, to_list_kv)
@@ -296,6 +311,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_multi_pages(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         async with self.create_aad_client(appconfiguration_endpoint_string) as client:
             # create PAGE_SIZE+1 configuration settings to have at least two pages
             try:
@@ -336,6 +353,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_configuration_settings_only_accepttime(self, appconfiguration_endpoint_string, **kwargs):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         recorded_variables = kwargs.pop("variables", {})
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         exclude_today = await self.convert_to_list(
@@ -354,6 +373,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_revisions_key_label(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         to_list = self.create_config_setting()
         items = await self.convert_to_list(
@@ -366,6 +387,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_revisions_only_label(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         items = await self.convert_to_list(self.client.list_revisions(label_filter=LABEL))
         assert len(items) >= 1
@@ -375,6 +398,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_revisions_key_no_label(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         items = await self.convert_to_list(self.client.list_revisions(key_filter=KEY))
         assert len(items) >= 1
@@ -384,6 +409,8 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
     @app_config_aad_decorator_async
     @recorded_by_proxy_async
     async def test_list_revisions_fields(self, appconfiguration_endpoint_string):
+        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
+        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
         await self.set_up(appconfiguration_endpoint_string, is_aad=True)
         items = await self.convert_to_list(
             self.client.list_revisions(key_filter="*", label_filter=LABEL, fields=["key", "content_type"])
@@ -574,15 +601,15 @@ class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
             assert temp["conditions"] == set_flag_value["conditions"]
 
             changed_flag.value = json.dumps({})
-            assert changed_flag.enabled == None
+            assert changed_flag.enabled == False
             temp = json.loads(changed_flag.value)
             assert temp["id"] == set_flag_value["id"]
-            assert temp["enabled"] == None
+            assert temp["enabled"] == False
             assert temp["conditions"] != None
             assert temp["conditions"]["client_filters"] == None
 
             set_flag.value = "bad_value"
-            assert set_flag.enabled == None
+            assert set_flag.enabled == False
             assert set_flag.filters == None
             assert set_flag.value == "bad_value"
 
